@@ -97,7 +97,7 @@ sub stop {
     my $s    = $self->{state};
     my $sdev = $self->{sdev_current};
     unless ($p->{leave_in_drain_mode}) {
-        MogileFS::Device->of_devid($sdev)->set_state('alive') if $sdev;
+        Mgd::get_store()->set_device_state($sdev, 'alive') if $sdev;
     }
     $s->{time_stopped} = time();
 }
@@ -211,7 +211,7 @@ sub next_fids_to_rebalance {
     # If we're not working against a source device, discover one
     my $sdev = $self->_find_source_device($state->{source_devs});
     return undef unless $sdev;
-    $sdev = MogileFS::Device->of_devid($sdev);
+    $sdev = Mgd::device_factory()->get_by_id($sdev);
     my $filtered_destdevs = $self->filter_dest_devices($devs);
 
     croak("rebalance cannot find suitable destination devices")
@@ -309,7 +309,7 @@ sub filter_source_devices {
  
     my @sdevs = ();
     for my $dev (@$devs) {
-        next unless $dev->exists && $dev->can_delete_from;
+        next unless $dev->can_delete_from;
         my $id = $dev->id;
         if (@{$policy->{from_devices}}) {
             next unless grep { $_ == $id } @{$policy->{from_devices}};
@@ -361,7 +361,7 @@ sub _finish_source_device {
     # Unless the user wants a device to never get new files again (sticking in
     # drain mode), return to alive.
     unless ($policy->{leave_in_drain_mode}) {
-        MogileFS::Device->of_devid($sdev)->set_state('alive');
+        Mgd::get_store()->set_device_state($sdev, 'alive');
     }
     push @{$state->{completed_devs}}, $sdev;
 }
@@ -392,7 +392,7 @@ sub _find_source_device {
             }
         }
         # Must mark device in "drain" mode while we work on it.
-        MogileFS::Device->of_devid($sdev)->set_state('drain');
+        Mgd::get_store()->set_device_state($sdev, 'drain');
         $state->{sdev_limit} = $limit;
     }
 
@@ -434,7 +434,7 @@ sub filter_dest_devices {
 
     my @ddevs = ();
     for my $dev (@devs) {
-        next unless $dev->exists && $dev->should_get_new_files;
+        next unless $dev->should_get_new_files;
         my $id = $dev->id;
         my $hostid = $dev->hostid;
 
