@@ -11,7 +11,7 @@ use fields ('psock',              # socket for parent/child communications
             'queue_todo',         # aref of hrefs of work sent from parent
             );
 
-use MogileFS::Util qw(error eurl decode_url_args);
+use MogileFS::Util qw(error eurl decode_url_args apply_state_events);
 use MogileFS::Server;
 
 use vars (
@@ -188,39 +188,6 @@ sub parent_ping {
     }
 }
 
-sub broadcast_device_writeable {
-    $_[0]->_broadcast_state("device", $_[1], "writeable");
-}
-sub broadcast_device_readable {
-    $_[0]->_broadcast_state("device", $_[1], "readable");
-}
-sub broadcast_device_unreachable {
-    $_[0]->_broadcast_state("device", $_[1], "unreachable");
-}
-sub broadcast_host_reachable {
-    $_[0]->_broadcast_state("host", $_[1], "reachable");
-}
-sub broadcast_host_unreachable {
-    $_[0]->_broadcast_state("host", $_[1], "unreachable");
-}
-
-sub _broadcast_state {
-    my ($self, $what, $whatid, $state) = @_;
-    if ($what eq "host") {
-        MogileFS::Host->of_hostid($whatid)->set_observed_state($state);
-    } elsif ($what eq "device") {
-        MogileFS::Device->of_devid($whatid)->set_observed_state($state);
-    }
-    my $key = "$what-$whatid";
-    my $laststate = $self->{last_bcast_state}{$key};
-    my $now = time();
-    # broadcast on initial discovery, state change, and every 10 seconds
-    if (!$laststate || $laststate->[1] ne $state || $laststate->[0] < $now - 10) {
-        $self->send_to_parent(":state_change $what $whatid $state");
-        $self->{last_bcast_state}{$key} = [$now, $state];
-    }
-}
-
 sub invalidate_meta {
     my ($self, $what) = @_;
     return if $Mgd::INVALIDATE_NO_PROPOGATE;  # anti recursion
@@ -235,16 +202,6 @@ sub process_generic_command {
     my ($self, $lineref) = @_;
     return 0 unless $$lineref =~ /^:/;  # all generic commands start with colon
 
-    if ($$lineref =~ /^:state_change (\w+) (\d+) (\w+)/) {
-        my ($what, $whatid, $state) = ($1, $2, $3);
-        if ($what eq "host") {
-            MogileFS::Host->of_hostid($whatid)->set_observed_state($state);
-        } elsif ($what eq "device") {
-            MogileFS::Device->of_devid($whatid)->set_observed_state($state);
-        }
-        return 1;
-    }
-
     if ($$lineref =~ /^:shutdown/) {
         $$got_live_vs_die = 1 if $got_live_vs_die;
         exit 0;
@@ -255,11 +212,8 @@ sub process_generic_command {
         return 1;
     }
 
-    if ($$lineref =~ /^:invalidate_meta_once (\w+)/) {
-        local $Mgd::INVALIDATE_NO_PROPOGATE = 1;
-        # where $1 is one of {"domain", "device", "host", "class"}
-        my $class = "MogileFS::" . ucfirst(lc($1));
-        $class->invalidate_cache;
+    if ($$lineref =~ /^:monitor_events/) {
+        apply_state_events($lineref);
         return 1;
     }
 
@@ -279,18 +233,6 @@ sub process_generic_command {
         return 1;
     }
 
-    # :set_dev_utilization dev# 45.2 dev# 45.2 dev# 45.2 dev# 45.2 dev 45.2\n
-    # (dev#, utilz%)+
-    if (my ($devid, $util) = $$lineref =~ /^:set_dev_utilization (.+)/) {
-        my %pairs = split(/\s+/, $1);
-        local $MogileFS::Device::util_no_broadcast = 1;
-        while (my ($devid, $util) = each %pairs) {
-            my $dev = eval { MogileFS::Device->of_devid($devid) } or next;
-            $dev->set_observed_utilization($util);
-        }
-        return 1;
-    }
-
     # queue_name depth
     if ($$lineref =~ /^:queue_depth (\w+) (\d+)/) {
         $self->queue_depth($1, $2);
